/*******************************************************************************

Copyright (c) 1997-2004, Perforce Software, Inc.  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1.  Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

2.  Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTR
IBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL PERFORCE SOFTWARE, INC. BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/


/*******************************************************************************
 * Name		: p4result.cc
 *
 * Author	: Tony Smith <tony@perforce.com> or <tony@smee.org>
 *
 * Description	: C++ class for holding results of Perforce commands 
 *
 ******************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"

BEGIN_EXTERN_C()
#include "ext/standard/info.h"
END_EXTERN_C()

#ifdef ZEND_ENGINE_2
#include "zend_object_handlers.h"
#endif
#include <clientapi.h>
#include "p4result.h"

P4Result::P4Result()
{
    output = 0;
    errors = 0;
    warnings = 0;
    Reset();
}

P4Result::~P4Result()
{
    Reset();
}

void
P4Result::Reset()
{
    // Allow the old results to be reclaimed.
    if( output )	ZVAL_DELREF( output );
    if( errors )	ZVAL_DELREF( errors );
    if( warnings )	ZVAL_DELREF( warnings );

    output = 0;
    errors = 0;
    warnings = 0;

    // Make 'em
    MAKE_STD_ZVAL( output );
    MAKE_STD_ZVAL( warnings );
    MAKE_STD_ZVAL( errors );

    // Bake 'em
    array_init( output );
    array_init( warnings );
    array_init( errors );
}

//
// Append a string to the output array - duplicating the storage
// so that it's in Zend memory.
//
void
P4Result::AddOutput( const char *msg )
{
    add_next_index_string( output , (char *)msg, 1);
}

//
// Append another PHP object to the output array
//
void
P4Result::AddOutput( zval * out )
{
    add_next_index_zval( output , out );
}

//
// Append an error
//
void
P4Result::AddError( Error *e )
{
    StrBuf	m;
    e->Fmt( &m );

    int s;
    s = e->GetSeverity();

    // 
    // Empty and informational messages are pushed out as output as nothing
    // worthy of error handling has occurred. Warnings go into the warnings
    // list and the rest are lumped together as errors.
    //

    if ( s == E_EMPTY || s == E_INFO )
    {
	AddOutput( m.Text() );
	return;
    }

    if ( s == E_WARN )
	add_next_index_string( warnings , m.Text(), 1);
    else
	add_next_index_string( errors, m.Text(), 1 );
}

void
P4Result::AddError( const char *msg )
{
    add_next_index_string( errors, (char *)msg, 1 );
}

void
P4Result::AddWarning( const char *msg )
{
    add_next_index_string( warnings, (char *)msg, 1 );
}


int
P4Result::OutputCount()
{
    return Length( output );
}

int
P4Result::ErrorCount()
{
    return Length( errors );
}

int
P4Result::WarningCount()
{
    return Length( warnings );
}


int
P4Result::Length( zval *ary )
{
    return zend_hash_num_elements( HASH_OF( ary ) );
}

